#include <string.h>
#include "board.h"
#include "configuration.h"
#include "cmdline.h"
#include "packet.h"
#include "buffer.h"
#include "nb.h"


#define PACKET_INFO_VERSION	1	/* Version of packet structure */


Buffer_TypeDef PACKET_RXMSG, PACKET_TXMSG;

void PACKET_Init(void)
{
	BUFFER_Create(&PACKET_TXMSG, 16);	//gsm message buffer
}

uint16_t PACKET_getCrc16(uint8_t *buffer, uint32_t offset, uint32_t bufLen, uint16_t polynom, uint16_t preset)
{
	preset &= 0xFFFF;
	polynom &= 0xFFFF;
	uint16_t crc = preset;
	uint32_t i, j;
	uint16_t data;
	for (i = 0; i < bufLen; i++)
	{
		data = buffer[i + offset] & 0xFF;
		crc ^= data;
		for (j = 0; j < 8; j++)
		{
			if ((crc & 0x0001) != 0) crc = (crc >> 1) ^ polynom;
			else crc = crc >> 1;
		}
	}
	return crc & 0xFFFF;
}

/*
 *
 * in: uint32_t mask
 * mask bit
 * 0 - restart flag
 */

void PACKET_Create(uint32_t mask)
{
	BufferItem_TypeDef data;
	uint32_t timestamp = 0;//(uint32_t)RTC_GetTimestamp();
	uint16_t crc, i, elements, elements_pos;

	data.len = 0;
	data.data[data.len++] = PACKET_INFO_VERSION;	//0
	data.data[data.len++] = 0x00;					//1 - length
	data.data[data.len++] = 0x00;					//2 - length
	data.data[data.len++] = (timestamp >> 24)&0xFF;
	data.data[data.len++] = (timestamp >> 16)&0xFF;
	data.data[data.len++] = (timestamp >> 8)&0xFF;
	data.data[data.len++] = (timestamp)&0xFF;
	data.len += sprintf(&data.data[data.len],"%s", NBInfo.IMEI);
	elements_pos = data.len;
	data.data[data.len++] = 0;						//7 - Quantity of elements
	elements = 0;
	if (mask&PACKET_MASK_RESTART)
	{
		data.data[data.len++] = 0xFF;					//8 - Element Code
		data.data[data.len++] = 8;						//9 - Element Length
		data.data[data.len++] = 0x00;					//10 - Element ID 0=DBG
		data.data[data.len++] = 'R';					//11 - Element Value
		data.data[data.len++] = 'E';					//12 - Element Value
		data.data[data.len++] = 'S';					//13 - Element Value
		data.data[data.len++] = 'T';					//14 - Element Value
		data.data[data.len++] = 'A';					//15 - Element Value
		data.data[data.len++] = 'R';					//16 - Element Value
		data.data[data.len++] = 'T';					//17 - Element Value
		elements++;
	}
	else if (mask&PACKET_MASK_DCLICK)
	{
		data.data[data.len++] = 0x01;					//8 - Element Type
		data.data[data.len++] = 2;						//9 - Element Length
		data.data[data.len++] = 0x02;					//10 - Element ID
		data.data[data.len++] = NBInfo.RxLevel;	    	//11 - Element Value
		elements++;
	}
	else if (mask&PACKET_MASK_CLICK)
	{
		data.data[data.len++] = 0xFF;					//8 - Element Code
		data.data[data.len++] = 6;						//9 - Element Length
		data.data[data.len++] = 0x00;					//10 - Element ID 0=DBG
		data.data[data.len++] = 'C';					//11 - Element Value
		data.data[data.len++] = 'L';					//12 - Element Value
		data.data[data.len++] = 'I';					//13 - Element Value
		data.data[data.len++] = 'C';					//14 - Element Value
		data.data[data.len++] = 'K';					//15 - Element Value
		elements++;
	}
	else if (mask&PACKET_MASK_FORWARD_FLOW)
	{
		data.data[data.len++] = 0x03;					//8 - Element Code
		data.data[data.len++] = 5;						//9 - Element Length
		data.data[data.len++] = 0x01;					//10 - Element ID 1=Flow Counter
		data.data[data.len++] = 0xDE;					//11 - Element Value
		data.data[data.len++] = 0xAD;					//12 - Element Value
		data.data[data.len++] = 0xBE;					//13 - Element Value
		data.data[data.len++] = 0xEF;					//14 - Element Value
		elements++;
	}
	data.data[elements_pos] = elements;		//7 - Quantity of elements
	data.data[1] = ((data.len-3) >> 8) & 0xff;		//1 - length
	data.data[2] = ((data.len-3)) & 0xff;			//2 - length
	crc = PACKET_getCrc16(data.data, 3, data.len-3, 0xA001, 0);
	data.data[data.len++] = (crc >> 8) & 0xFF;	//CRC
	data.data[data.len++] = (crc) & 0xFF;		//CRC
	printf("TX: ");
	for (i=0; i<data.len; i++)
	{
		printf("%02x ", data.data[i]);
	}
	printf(" \r\n");
	BUFFER_Add(&PACKET_TXMSG, &data);
}
